﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows.Threading;
using Infragistics.Collections;

namespace WpfApplication1
{
    public class MainWindowViewModel : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged Implementation
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        protected void SetProperty<T>(ref T field, T value, string propertyName)
        {
            field = value;
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        private FakeEKGData _ekgData;
        public FakeEKGData EKGData
        {
            get { return _ekgData; }
            set
            {
                if (_ekgData != value)
                {
                    _ekgData = value;
                    OnPropertyChanged("EKGData");
                }
            }
        }

        public MainWindowViewModel()
        {
            // Tried 4000 data points with a 250 clear gap and on my tablet the cpu was between 30 and 40%
            // Due to the way this code updates the points though, it doesn't update as fast as yours I believe.
            //EKGData = new FakeEKGData(4000, 250);

            // I tried 200 data points instead and it still looks like an EEG, and performs much better because of
            // the fewer data points.
            EKGData = new FakeEKGData(200, 10);
        }
    }

    public class FakeEKGData : ObservableCollection<FakeEKGDataPoint>
    {
        private DispatcherTimer _timer = new DispatcherTimer();
        private int _currIndex = 0;
        private int _clearWidth = 0;

        private double _currFakeValue = 10.0;

        public FakeEKGData() : this(100, 5)
        {

        }

        public FakeEKGData(int length, int clearWidth)
        {
            _clearWidth = clearWidth;
            for (var i = 0; i < length; i++)
            {
                Add(new FakeEKGDataPoint() { TimeValue = "", Value = double.NaN });
            }

            _timer.Interval = TimeSpan.FromMilliseconds(50);
            _timer.Tick += new EventHandler(_timer_Tick);
            _timer.Start();
        }

        void _timer_Tick(object sender, EventArgs e)
        {
            _currFakeValue = GetNextFakeValue(_currFakeValue);
            var now = DateTime.Now;
            this[_currIndex] = new FakeEKGDataPoint()
            {
                TimeValue = DateTime.Now.AddSeconds(8.0).TimeOfDay.ToString("hh\\:mm\\:ss"),
                Value = _currFakeValue
            };
            for (var i = 1; i < _clearWidth + 1; i++)
            {
                if (_currIndex + i > this.Count - 1)
                {
                    break;
                }
                if (!double.IsNaN(this[_currIndex + i].Value))
                {
                    this[_currIndex + i] = new FakeEKGDataPoint()
                    {
                        TimeValue = now.AddSeconds((double)i * .03).TimeOfDay.ToString("hh\\:mm\\:ss"),
                        Value = double.NaN
                    };
                }
            }

            _currIndex++;
            if (_currIndex > this.Count - 1)
            {
                _currIndex = 0;
            }
        }

        private Random _rand = new Random();

        private double GetNextFakeValue(double _currFakeValue)
        {
            if (_rand.NextDouble() > .5)
            {
                var ret = _currFakeValue += _rand.NextDouble() * 2.0;
                if (ret > 15)
                {
                    ret = 15;
                    ret = _currFakeValue -= _rand.NextDouble() * 2.0;
                }
                return ret;
            }
            else
            {
                var ret = _currFakeValue -= _rand.NextDouble() * 2.0;
                if (ret < -15)
                {
                    ret = -15;
                    ret = _currFakeValue += _rand.NextDouble() * 2.0;
                }
                return ret;
            }
        }
    }

    public class FakeEKGDataPoint
    {
        public string TimeValue { get; set; }
        public double Value { get; set; }
    }

}
